a {
  color: #707098;
}
a:hover {
  color: #404058;
}

#header {
  background-color: #444;
  padding: 8px;
}
#header a {
  color: #f8f8f8;
  margin: 0;
  padding: 0;
  text-decoration: none;
}
.sign-out-form {
  padding: 3px 20px;
  clear: both;
}
.title {
  margin: 0;
  padding: 0;
  height: 96px;
}
.tagline {
  height: 16px;
  font-size: 16px;
  line-height: 16px;
}

nav ul {
  width: 100%;
  background-color: #d0d0e8;
}
nav li a,
.bottom-nav a {
  color: #444;
}

.flash {
  margin: 8px 0;
  padding: 0;
}
.flash li {
  margin: 4px 0;
  padding: 4px;
  list-style: none;
}

.bottom-nav {
  background-color: #d0d0e8;
}

footer {
  font-size: 85%;
  text-align: center;
  margin-top: 8px;
}

/* Bootstrap Overrides */

.btn-primary {
  color: #444;
  background-color: #d0d0e8;
  border-color: #444;
}
.btn-primary:hover {
  color: #222;
  background-color: #b0b0c8;
  border-color: #111;
}

.btn-link {
  color: #707098;
}
.btn-link:hover {
  color: #404058;
}

.btn-danger {
  color: #644;
  background-color: #e8d0d0;
  border-color: #888;
}
.btn-danger:hover {
  color: #422;
  background-color: #c8b0b0;
  border-color: #555;
}
